function [vs_delay, vs_gain, num_vs] = earlyEchos(roomSize, sourcePos, listenerPos)

% [vs_delay, vs_gain] = earlyEchos(roomSize, sourcePos, listenerPos)
%
% generate early echo reverb response based on room geometry and
% source and listener positions
% roomSize = [length width height]  (in meters)
% sourcePos = [elevation azimuth distance] (relative to listener)
% listenerPos = [x y z] (pos of listener's head) (in meters)
%
% coordinate system definitions
% looking from above:
% lower left corner of room is (0,0,0)
% upper right corner of room is (W,L,H)
%
%  (0,L,H) +-----------+ (W,L,H)
%          |           |
%        L |           |
%        ^ |           |
%        | |           |
%        | |           |
%        | |           |
%        | |           |
%          +-----------+
%   (0,0,H)  -----> W    (W,O,H)

% compute sourcePos [x y z] from [el az dist]

listenerX = listenerPos(1);
listenerY = listenerPos(2);
listenerZ = listenerPos(3);

roomX = roomSize(1);
roomY = roomSize(2);
roomZ = roomSize(3);

az = sourcePos(2)*pi/180;  
el = sourcePos(1)*pi/180;
dist = sourcePos(3);       % meters

horiz_dist = dist*cos(el);
X_distance = horiz_dist*sin(az);  % negative for negative angles
Y_distance = horiz_dist*cos(az);  % negative for |az| bet 90 and 180
Z_distance = dist*sin(el);        % negative for negative angles

sourceX = X_distance + listenerX;
sourceY = Y_distance + listenerY;
sourceZ = Z_distance + listenerZ;
 
if (( sourceX >  roomX) | (sourceX < 0) ) 
    disp('Error: Source wont fit in Room in X direction');
end
if (( sourceY >  roomY) | (sourceY < 0) ) 
    disp('Error: Source wont fit in Room in Y direction');
end
if (( sourceZ >  roomZ) | (sourceZ < 0) ) 
    disp('Error: Source wont fit in Room in Z direction');
end

sourcePos_xyz = [sourceX sourceY sourceZ];

num_vs = 16;
vs = zeros(num_vs,3);

% find primary virtual sources caused by
% floor
vs(1,:) = [sourceX sourceY -sourceZ];

% ceiling
vs(2,:) = [sourceX sourceY roomZ+(roomZ-sourceZ)];

% front wall 
vs(3,:) = [sourceX roomY+(roomY-sourceY) sourceZ];

% back wall 
vs(4,:) = [sourceX -sourceY sourceZ];

% left wall 
vs(5,:) = [-sourceX sourceY sourceZ];

% right wall 
vs(6,:) = [roomX+(roomX-sourceX) sourceY sourceZ]; 

% find secondary virtual sources caused by
% floor reflects ceiling vs
vs(7,:) = [sourceX sourceY -vs(2,3)];

% ceiling reflects floor vs
vs(8,:) = [sourceX sourceY abs(vs(1,3))+roomZ];

% front wall reflects back wall vs
vs(9,:) = [sourceX abs(vs(4,2))+roomY sourceZ];

% back wall reflects front wall vs
vs(10,:) = [sourceX -vs(3,2) sourceZ];

% left wall reflects right wall vs
vs(11,:) = [-vs(6,1) sourceY sourceZ];

% right wall reflects left wall vs
vs(12,:) = [roomX+abs(vs(5,1)) sourceY sourceZ]; 

% find corner virtual sources 
% front right 
vs(13,:) = [roomX+(roomX-sourceX) roomY+(roomY-sourceY) sourceZ];

% front left
vs(14,:) = [-sourceX roomY+(roomY-sourceY) sourceZ];

% back right
vs(15,:) = [roomX+(roomX-sourceX) -sourceY sourceZ];

% back left
vs(16,:) = [-sourceX -sourceY sourceZ];

% display room size, source position, virtual source location
%fprintf('Room Size:          L = %2.4f   W = %2.4f   H = %2.4f\n', roomX, roomY, roomZ)
%fprintf('Listener Loc:       az = %2.4f  el = %2.4f  dist = %2.4f\n', sourcePos(2), sourcePos(1), sourcePos(3));
%fprintf('Listener Loc:       x = %2.4f   y = %2.4f   z = %2.4f\n', listenerX, listenerY, listenerZ)  
%fprintf('Source Loc:         x = %2.4f   y = %2.4f   z = %2.4f\n', sourceX, sourceY, sourceZ)  

for i=1:num_vs
   %fprintf('Virtual Source %d:   x = %2.4f   y = %2.4f   z = %2.4f\n',i, vs(i,1), vs(i,2), vs(i,3))
end

% compute distance between vs's and listener
% compute delay time between vs's and listener
% compute gain (attenuation) of vs's to listener = 1/d

c = 340; % speed of sound in meters/sec

for i=1:num_vs
   vs_dist(i) = xyz_dist(vs(i,:), listenerPos);
   vs_delay(i) = vs_dist(i)/c;
   vs_gain(i) = 1/vs_dist(i);
end

vs_dist;
vs_delay;
vs_gain;
num_vs;
